--[[---------------------------------------------------------------------------------

«Data-URL-Encode.lua» Version 0.003b

Convert any image to base64 encoded inline using data:URL scheme in HTML or CSS file.
Set text cursor under filename and press «Ctrl+R».

Convert any selected base64 data to file.
Select base64 data and press «Ctrl+Alt+R».

Package:

  /Modules
    /ltn12
      init.lua
    /mime
      core.dll
      init.lua
    base64.lua
  /Scripts
    Data-URL-Encoder.v.0.003b.lua

Linked pages:
  http://forum.farmanager.com/viewtopic.php?f=60&p=141277

--]]---------------------------------------------------------------------------------

local base64 = require 'base64'

-------------------------------------------------------------------------------------

function readfile (filename, flags)
  local file = io.open(filename, 'r'..flags)
  if not file then
    return '', false, 'the file cannot be opened'
  end
  local buffer = file:read('*a')
  file:close()
  return buffer, true, ''
end

function writefile (filename, contents, flags)
  local file = io.open(filename, 'w'..flags)
  if not file then return false end
  file:write(contents)
  file:close()
  return true
end

local signatureBySuffix  = {
  ttf     = '\0\1\0\0',
  otf     = 'OTTO',
  woff    = 'wOFF',
  woff2   = 'wOF2',
  jpg     = '\255\216\255',
  png     = '\137PNG\13\10\26\10',
  gif     = 'GIF',
  ico     = '\0\0\1\0',
  swf     = 'FWS',
  swc     = 'CWS',
  bmp     = 'BM',
  psd     = '8BPS',
  jpc     = '\255\79\255',
  jp2     = '\0\0\0\12jP\32\32\13\10\135\10',
  iff     = 'FORM',
  tiff_ii = 'II\42\0',
  tiff_mm = 'MM\0\42',
}

local mimeTypeBySuffix = {
  ttf     = 'application/x-font-truetype',
  otf     = 'application/x-font-opentype',
  woff    = 'application/font-woff',
  woff2   = 'application/font-woff2',
  eot     = 'application/vnd.ms-fontobject',
  jpg     = 'image/jpeg',
  png     = 'image/png',
  gif     = 'image/gif',
  ico     = 'image/vnd.microsoft.icon',
  swf     = 'application/x-shockwave-flash',
  swc     = 'application/x-shockwave-flash',
  bmp     = 'image/x-ms-bmp',
  psd     = 'image/psd',
  jpc     = 'application/octet-stream',
  jp2     = 'image/jp2',
  iff     = 'image/iff',
  tiff_ii = 'image/tiff',
  tiff_mm = 'image/tiff',
  unknown = 'application/octet-stream',
}

local suffixByMimeType = {}

for k, v in pairs(mimeTypeBySuffix) do suffixByMimeType[v] = k end

local function getSuffixByMimeType (mimetype)
  return '.' .. (suffixByMimeType[mimetype] or 'xxx')
end

local function getFormat (startChunk)
  for format, sign in pairs(signatureBySuffix) do
    if startChunk:sub(1, sign:len()) == sign then
      return format
    end
  end
  return 'unknown'
end

local function getMimeType (name)
  local mime = ''
  if type(name) == 'string' then
    local handle, err = io.open(name, 'rb')
    if handle then
      local startChunk = handle:read(16)
      if startChunk then
        mime = mimeTypeBySuffix[getFormat(startChunk)]
      end
      handle:close()
    end
  end
  return mime
end

local function getQuotedValueUnderCursor (Line, CurPos)
  local LineLen = Line:len()
  local ReversPos = 2 + LineLen - CurPos
  local ReversStartPos = Line:reverse():find('[(%\'"]', ReversPos)
  if not ReversStartPos then return end
  local StartPos = 1 + LineLen - ReversStartPos
  local OpenQuote = Line:sub(StartPos, StartPos)
  local CloseQuote = OpenQuote
  if OpenQuote == '(' then CloseQuote = ')' end
  local EndPos = Line:find(CloseQuote, CurPos)
  if not EndPos then return end
  local SearchedValue = Line:sub(StartPos + 1, EndPos - 1)
  SearchedValue = SearchedValue:gsub('[(%\'"]', '')
  if SearchedValue == '' then return end
  return true, {
    Value = SearchedValue,
    StartPos = StartPos,
    EndPos = EndPos,
    From = StartPos + 1,
    To = EndPos - StartPos - 1,
    OpenQuote = OpenQuote,
    CloseQuote = CloseQuote,
  }
end

local function DataURLEncoder ()

  local EditorID = editor.GetInfo().EditorID
  local filename

  if Editor.SelValue and Editor.SelValue ~= '' then
    filename = Editor.SelValue
  else
    local ok, data = getQuotedValueUnderCursor(Editor.Value, Editor.CurPos)
    if not ok then return end
    filename = data.Value
    editor.Select(EditorId, 1, Editor.CurLine, data.From, data.To, 1)
  end

  local fullname = editor.GetFileName(EditorId)..'/../'..filename
  local mime = getMimeType(fullname)
  local content, ok = readfile(fullname, 'b')

  if not ok then
    msgbox ('Error opening file', 'The file „'..filename..'“ cannot be read.')
    return
  end

  local b64data = tostring(base64.encode(content))

  editor.DeleteBlock(EditorID)

  print('data:' .. mime .. ';base64,' .. b64data)

end

local function DataURLDecoder ()

  local EditorID = editor.GetInfo().EditorID
  local value

  if Editor.SelValue and Editor.SelValue ~= '' then
    value = Editor.SelValue
  else
    local ok, data = getQuotedValueUnderCursor(Editor.Value, Editor.CurPos)
    if not ok then return end
    value = data.Value
    editor.Select(EditorId, 1, Editor.CurLine, data.From, data.To, 1)
  end

  local _, len, mimetype = value:find('^data:([%w-]+/[%w-]+);base64,')

  value = value:sub(len+1)

  local decoded = base64.decode(value)

  if not decoded then return end

  local filename = far.InputBox(nil, 'Decode base64',
    'Save „' .. mimetype .. '“ file as:', 'DataURLDecoder:files',
    APanel.Path .. '\\untitled' .. getSuffixByMimeType(mimetype)
  )

  if filename then
    writefile(filename, decoded, 'b')
  end

end

-------------------------------------------------------------------------------------

Macro {
  description = 'Data URL Encoder (0.003b) → DataURLEncoder';
  area = 'Editor';
  key  = 'CtrlR';
  action = DataURLEncoder;
}

Macro {
  description = 'Data URL Encoder (0.003b) → DataURLDecoder';
  area = 'Editor';
  key  = 'CtrlAltR';
  action = DataURLDecoder;
}



